/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.common.binaryfile;

import java.io.InputStream;
import java.util.function.BiFunction;
import java.util.function.Consumer;

import io.grpc.stub.CallStreamObserver;
import lombok.extern.log4j.Log4j2;

@Log4j2
class StreamSharingFileSender<Q, S> extends StreamingFileSender<Q, S> {

	private final CallStreamObserver<Q> requestObserver;
	private final Consumer<Runnable> onReadyHandlerRegistrar;

	StreamSharingFileSender(BiFunction<byte[], Integer, Q> chunkBuilder, InputStream inputStream, CallStreamObserver<Q> requestObserver, Consumer<Runnable> onReadyHandlerRegistrar) {
		super(chunkBuilder, inputStream);

		this.requestObserver = requestObserver;
		this.onReadyHandlerRegistrar = onReadyHandlerRegistrar;
	}

	@Override
	public StreamSharingFileSender<Q, S> send() {
		LOG.debug("Register onReadyHandler and start sending File.");
		onReadyHandlerRegistrar.accept(this::sendNext);
		return this;
	}

	@Override
	void communicateEndOfTransfer() {
		sendEndOfFile();
		getResultFuture().complete(null);
	}

	private void sendEndOfFile() {
		sendChunk(new byte[0], -1);
	}

	@Override
	protected CallStreamObserver<Q> getRequestObserver() {
		return requestObserver;
	}
}
