/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.common.binaryfile;

import static org.assertj.core.api.Assertions.*;

import java.io.InputStream;
import java.util.function.BiFunction;
import java.util.function.Consumer;
import java.util.function.Function;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;

import de.ozgcloud.common.binaryfile.BinaryFileTestFactory.TestRequestType;
import de.ozgcloud.common.binaryfile.BinaryFileTestFactory.TestResponseType;
import io.grpc.stub.CallStreamObserver;
import io.grpc.stub.StreamObserver;

class GrpcFileUploadUtilsTest {

	@Mock
	private BiFunction<byte[], Integer, TestRequestType> chunkBuilder;
	@Mock
	private InputStream inputStream;
	@Mock
	private Function<StreamObserver<TestResponseType>, CallStreamObserver<TestRequestType>> reqObserverBuilder;
	@Mock
	private CallStreamObserver<TestRequestType> requestObserver;
	@Mock
	private Consumer<Runnable> onReadyHandlerRegistrar;

	@Nested
	class TestCreateSender {

		@Test
		void shouldReturnInstanceOfStreamExclusiveFileSender() {
			var createdSender = GrpcFileUploadUtils.createSender(chunkBuilder, inputStream, reqObserverBuilder);

			assertThat(createdSender).isInstanceOf(StreamExclusiveFileSender.class);
		}
	}

	@Nested
	class TestCreateStreamExclusiveFileSender {

		@Test
		void shouldReturnInstanceOfStreamExclusiveFileSender() {
			var createdSender = GrpcFileUploadUtils.createStreamExclusiveFileSender(chunkBuilder, inputStream, reqObserverBuilder);

			assertThat(createdSender).isInstanceOf(StreamExclusiveFileSender.class);
		}
	}

	@Nested
	class TestCreateStreamSharingSender {

		@Test
		void shouldReturnInstanceOfStreamSharingSender() {
			var createdSender = GrpcFileUploadUtils.createStreamSharingSender(chunkBuilder, inputStream, requestObserver, onReadyHandlerRegistrar);

			assertThat(createdSender).isInstanceOf(StreamSharingFileSender.class);
		}
	}
}