/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.common.binaryfile;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.InputStream;
import java.util.function.BiFunction;
import java.util.function.Consumer;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.common.binaryfile.BinaryFileTestFactory.TestRequestType;
import de.ozgcloud.common.binaryfile.BinaryFileTestFactory.TestResponseType;
import io.grpc.stub.CallStreamObserver;

class StreamSharingFileSenderTest {

	@Mock
	private BiFunction<byte[], Integer, TestRequestType> chunkBuilder;
	@Mock
	private InputStream inputStream;
	@Mock
	private CallStreamObserver<TestRequestType> requestObserver;
	@Mock
	private Consumer<Runnable> onReadyHandlerRegistrar;
	@Spy
	@InjectMocks
	private StreamSharingFileSender<TestRequestType, TestResponseType> fileSender;

	@Nested
	class TestSend {

		@Captor
		private ArgumentCaptor<Runnable> onReadyHandlerCaptor;

		@Test
		void shouldRegisterOnReadyHandler() {
			doNothing().when(fileSender).sendNext();

			fileSender.send();

			verify(onReadyHandlerRegistrar).accept(onReadyHandlerCaptor.capture());
			verifyCallsSendNext(onReadyHandlerCaptor.getValue());
		}

		@Test
		void shouldReturnThis() {
			var obj = fileSender.send();

			assertThat(obj).isSameAs(fileSender);
		}

		private void verifyCallsSendNext(Runnable runnable) {
			runnable.run();
			verify(fileSender).sendNext();
		}
	}

	@Nested
	class TestCommunicateEndOfTransfer {

		@Test
		void shouldSendEmptyChunk() {
			fileSender.communicateEndOfTransfer();

			verify(fileSender).sendChunk(new byte[0], -1);
		}

		@Test
		void shouldCompleteResultFuture() {
			fileSender.communicateEndOfTransfer();

			assertThat(fileSender.getResultFuture().isDone()).isTrue();
		}
	}
}
