/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.common.logging;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import jakarta.interceptor.InvocationContext;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockedStatic;

class OzgCloudLoggingInterceptorTest {

	private final OzgCloudLoggingInterceptor interceptor = new OzgCloudLoggingInterceptor();

	@DisplayName("Logging")
	@Nested
	class TestLogging {

		@Mock
		private InvocationContext context;

		private MockedStatic<AspectLoggingUtils> loggingUtilMock;

		@BeforeEach
		void mockLoggingUtils() {
			loggingUtilMock = mockStatic(AspectLoggingUtils.class);
		}

		@AfterEach
		public void after() {
			loggingUtilMock.close();
		}

		@Nested
		class TestProceedSuccessful {

			private Object nextChainMethod = null;

			@Test
			void shouldLog() throws Exception {
				when(context.proceed()).thenReturn(nextChainMethod);

				interceptor.logging(context);

				loggingUtilMock.verify(() -> AspectLoggingUtils.log(context));
			}

			@Test
			void shouldLogReturnValue() throws Exception {
				when(context.proceed()).thenReturn(nextChainMethod);

				interceptor.logging(context);

				loggingUtilMock.verify(() -> AspectLoggingUtils.logReturnValue(context, nextChainMethod));
			}

			@Test
			void shouldReturnValue() throws Exception {
				when(context.proceed()).thenReturn(nextChainMethod);

				var result = interceptor.logging(context);

				assertThat(result).isEqualTo(nextChainMethod);
			}
		}

		@Nested
		class TestProceedFailing {

			@Test
			void shouldLogException() throws Exception {
				var exception = new Exception();
				when(context.proceed()).thenThrow(exception);

				try {
					interceptor.logging(context);
				} catch (Exception e) {
					loggingUtilMock.verify(() -> AspectLoggingUtils.logException(context, e));
				}
			}

			@Test
			void shouldThrowException() throws Exception {
				var exception = new Exception();
				when(context.proceed()).thenThrow(exception);

				assertThatExceptionOfType(Exception.class).isThrownBy(() -> interceptor.logging(context));
			}
		}
	}
}
